#! /bin/csh -f
#
#  Automatically generated script 
#  for extracting one of FP from an MTZ
#
set nawk = /bin/awk
$nawk 'BEGIN{print}' >& /dev/null
if($status) set nawk = awk
alias nawk $nawk
#
set infile   = ./mtz/localscaled.mtz
set outfile  = unmerged.mtz
set runfile  = ./runlist.txt
#
set tempfile = /tmp/jamesh/scaler_temp29967.extract
#
# Default to all data (useful? )
if(! $?wave) set wave     = "all"
set hires    = 2.11
#
# read command-line arguments
goto Setup
############################
#
# usage: ./scripts/extract.com [wavename] [high_RES]
#    eg: ./scripts/extract.com FP 2.11
#        means extract frames from FP to 2.11 A
#
############################
ReturnFromSetup:
######################################################################
#
# Now run SCALA (to add partials, not scale data)
#
######################################################################
rm -f ${tempfile}unmerged.mtz >& /dev/null
scala hklin $infile  hklout ${tempfile}unmerged.mtz \
  scales   ${tempfile}scales \
  rogues   ${tempfile}rogues \
  normplot ${tempfile}norm   \
  anomplot ${tempfile}anom  << EOF-scale
# save disk space
#NODUMP
#ANALYSE NOPLOT
DNAME $wave
PNAME $wave

title "unmerged $wave from Scaler Elves"

# resolution range
RESOLUTION $hires 121.268

@${tempfile}runlist

# global SCALA cards
SDCORR 0.75 2 0.02
INTENSITIES SCALE_PARTIALS 0.75 

# treatment of partials	    # default, sum partials, and use result in analysis
#final fulls		    # use fulls only in analysis
#final scale_partials 0.5   # treat partials with fraction recorded > 0.5 as fulls in mergeing

output unmerged original    # don't merge to ASU, but add partials
noscale			    # should apply old scales!
onlymerge
#scales batch

# changing this to "off" will NOT affect output Is
anomalous on

EOF-scale
##############################################################################
if($status) then
    echo "SCALA crashed! "
    # check to see if file was produced anyway
    echo "go" | mtzdump hklin ${tempfile}unmerged.mtz >& /dev/null
    if($status) then
	# file was no good
	rm -f ${tempfile}unmerged.mtz >& /dev/null
    else
	echo "...but seems to have produced the output data file before it did."
	echo "YOU SHOULD CHECK AND SEE IF THIS IS OKAY! "
    endif
endif

##############################################################################
# take existence of output file as signal of a sucessful run
if(-e ${tempfile}unmerged.mtz) then
    mv ${tempfile}unmerged.mtz ${outfile}
    echo "unmerged $wave data is in ${outfile}"
else
    echo "extraction failed!  See above for why."
    exit 9
endif

# clean up after scala
rm -f ${tempfile}rogues        >& /dev/null
rm -f ${tempfile}runlist       >& /dev/null
rm -f ${tempfile}scales        >& /dev/null
rm -f ${tempfile}norm	        >& /dev/null
rm -f ${tempfile}anom	        >& /dev/null

exit

##############################################################################
Setup:
#
# Run List passed down from Scaler Elves
if(! -e "$runfile" ) then
    # generate file containing list of scala runs
    cat << EOF-runs >! $runfile
# wavelength FP : 0.97973 A = 12654.9 eV
# 11000 +     1 to   100 in ../raw.mtz
run     1 11001 to 11100

EOF-runs
endif
#
# process command-line arguments
foreach arg ( $* )
    # define the input file
    if(("$arg" =~ *.mtz)&&(-e "$arg")) then
	set infile = "$arg"
    else
    
	# look for wavelength name
	set temp = `grep "wavelength" $runfile | grep " $arg " `
	if("$temp" != "") then
	    # must be the wavelength user wants
	    set wave = "$arg"
	else
	
	    # look for new resolution limit
	    set temp = `echo "$arg" | nawk '$1+0<10 && $1+0>0.4 {print $1+0}'`
	    if("$temp" != "") then
		# must be the resolution user wants
		set hires = "$temp"
	    endif
	
	endif
    endif
end
#
# Extract runs for desired wavelength
cat $runfile |\
nawk -v wave="$wave" '$2=="wavelength"{p=0} $0 ~ " " wave " "{p=1} p{print}' |\
cat >! ${tempfile}runlist
#
set temp = `cat ${tempfile}runlist | wc -l`
if($temp == 0) then
    echo "WARNING: no wavelength specified for extraction. "
    echo "extracting all data"
    echo "usage: $0 [FP] [2A]"
    echo ""
    echo "where:"
    echo "FP, etc.  is a wavelength name"
    echo "2A is the (optional) resolution cutoff"
    echo ""

    # default to all runs
    cat $runfile >! ${tempfile}runlist
endif

goto ReturnFromSetup

