#! /bin/csh -f
#
#   Reference set import script by Scaler Elves
#
#
set nawk = /bin/awk
$nawk 'BEGIN{print}' >& /dev/null
if($status) set nawk = awk
alias nawk $nawk
#
set infile    = ./mtz/rawdata.mtz
set reffile   = ./mtz/reference.mtz
set refdata   = ""
set ref_batch = 1
set outfile   = ./mtz/sorted_ref.mtz
#
set tempfile  = /tmp/jamesh/scaler_temp29967.get_ref
#
############################################################################
goto Setup
#
# scan command line for user options
#
Return_from_Setup:

##########################################################################
# Remove free-R flagged HKLs from the reference MTZ
#
##########################################################################
if($?ref_free) then
    # need to convert to text in order to exclude freeR
    mtz2various hklin $reffile hklout ${tempfile}.hkl << EOF-various | tee ${tempfile}log
    labin FP=$ref_set SIGFP=$ref_sig FREE=$ref_free
    OUTPUT XPLOR
    EXCLUDE FREER 0
EOF-various
    if($status) set BAD

    # prepare to read back in
    set CELL  = `nawk '/Cell Dimensions/{getline; getline; print}' ${tempfile}log`
    set SG    = `nawk '/Space group/{print $5}' ${tempfile}log`
    set SGnum = `nawk '/Space group/{print $NF+0}' ${tempfile}log`
    set SG    = `nawk -F "[\047]" '/Space group/{print $2}' ${tempfile}log`
    set SG    = `nawk -v num=$SGnum '$1==num && NF>5{print $4}' ${CLIBD}/symop.lib`
    rm -f ${tempfile}log >& /dev/null
    set temp = "F"
    if("$input" == mtzi) set temp = J
   
    # simplefy file
    cat ${tempfile}.hkl |\
    nawk '$1 ~ /^INDE/{split($0,w,"="); print $2, $3, $4, w[2]+0,w[3]+0}' |\
    cat >! ${tempfile}no_freeR.hkl
    
    # now read it back in...
    f2mtz hklin ${tempfile}no_freeR.hkl hklout ${tempfile}reference.mtz << EOF-f2mtz
    CELL $CELL
    SYMM $SG
    SKIP
    LABOUT H K L $ref_set $ref_sig
    CTYPO  H H H $temp Q
EOF-f2mtz
    if($status) set BAD
    
    # clean up
    rm -f ${tempfile}.hkl >& /dev/null
    rm -f ${tempfile}no_freeR.hkl >& /dev/null
else
    # symmetry with above
    cp $reffile ${tempfile}reference.mtz
endif

##########################################################################
# Reformat reference set for reinput 
#
##########################################################################
rotaprep hklin ${tempfile}reference.mtz  hklout ${tempfile}prep.mtz << eof-reprep
input $input
batch $ref_batch
labin $labin
DNAME ref
PNAME ref
eof-reprep
if($status) set BAD

# Put reference dataset together with raw data: $outfile
mtzutils hklin1 $infile hklin2 ${tempfile}prep.mtz  hklout $outfile << eof-utils
merge
eof-utils
if($status) set BAD

# clean up
rm -f ${tempfile}prep.mtz
rm -f ${tempfile}reference.mtz >& /dev/null


##########################################################################
if((-e $outfile)&&(! $?BAD)) then
    echo "$outfile now contains all raw data, plus "
    echo " ${ref_set}/${ref_sig} from ${reffile}, as a reference "
    if($?ref_free) then
        echo "excluding HKLs where $ref_free == 0"
    endif
else
    echo "import failed!  See above for why."
    exit 9
endif
##########################################################################
exit

Setup:
# now scan command line
foreach arg ( $* )
    # specific input file
    if($arg =~ *.mtz) then
        echo "HEAD" | mtzdump hklin $arg |\
        nawk '/Column Labels/{getline;getline;while(NF){printf "%s ", $0; getline};print ""}\
              /Column Types/ {getline;getline;while(NF){printf "%s ", $0; getline};print ""}' |\
	cat >! ${tempfile}.mtzdump
	grep "H K L" ${tempfile}.mtzdump >& /dev/null
        if($status) then
	    echo "WARNING: $arg is not an mtz file! "
	    rm -f ${tempfile}.mtzdump >& /dev/null
	    continue
        endif

	# check for multirecord mtz
	grep " BATCH I " ${tempfile}.mtzdump >& /dev/null
	if(! $status) then
	    # this must be the multirecord, raw-data file
	    set infile = $arg
	    rm -f ${tempfile}.mtzdump >& /dev/null
	    continue
	endif
	
	# this must be a single-record mtz (merged)
	set reffile = $arg
	cat ${tempfile}.mtzdump |\
	nawk 'NR==1{split($0,name)} NR==2{for(i=1;i<=NF;++i){print $i,name[i]}}' |\
	cat >! ${tempfile}.labels
	rm -f ${tempfile}.mtzdump >& /dev/null
	continue
    endif
end

if(! -e "$reffile") then
    echo "ERROR: reference mtz file does not exist! "
    rm -f ${tempfile}.labels >& /dev/null
    set BAD
    exit 9
endif


# see if user specified labels in the reference mtz
foreach arg ( $refdata $* )
    set temp = `nawk -v arg=$arg '$NF==arg' ${tempfile}.labels`
    if("$temp" =~ J*) then
    	set input = mtzi
    	set ref_set = "$temp[2]" 
    endif
    if("$temp" =~ F*) then
    	set input = mtzf
    	set ref_set = "$temp[2]" 
    endif
    if("$temp" =~ *FreeR_flag) then
    	set ref_free = "$temp[2]" 
    endif
    if("$temp" =~ Q*) then
    	set ref_sig = "$temp[2]" 
    endif
end
if(! $?ref_set) then
    set temp = `nawk '$1=="J" || $1=="F"{printf "%s ", $0;getline;print $NF}' ${tempfile}.labels | tail -1`
    if("$temp" =~ J*) then
    	set input = mtzi
    	set ref_set = "$temp[2]" 
    endif
    if("$temp" =~ F*) then
    	set input = mtzf
    	set ref_set = "$temp[2]" 
    endif
    if(! $?ref_set) then
	echo "ERROR: no data in $reffile"
	rm -f ${tempfile}labels >& /dev/null
	exit 9
    endif
endif
# set up the sigma (if not aready done)
if(! $?ref_sig) then
    set ref_sig = `nawk -v set=$ref_set '$1=="Q"{Q=$NF; if(nextq) exit} $NF==set{nextq=1} END{print Q}' ${tempfile}.labels`
endif
# use the FreeR flag (if it's there)
if(! $?ref_free) then
    set temp = `nawk 'tolower($NF) ~ /^freer/{print $NF}' ${tempfile}.labels`
    if("$temp" != "") set ref_free = $temp
endif
# set up rotaprep variables
if("$input" == "mtzi") then
    set labin = "I=$ref_set SIGI=$ref_sig"
endif
if("$input" == "mtzf") then
    set labin = "F=$ref_set SIGF=$ref_sig"
endif

echo "using $ref_set $ref_sig as the reference dataset"
if($?ref_free) then
    echo "(excluding free-R flagged hkls)"
endif

rm -f ${tempfile}.labels >& /dev/null

goto Return_from_Setup

