#! /bin/csh -f
#
#  Automatically generated multiwavelength SCALA script 
#  for local-scaling of FP together
#
set nawk = /bin/awk
$nawk 'BEGIN{print}' >& /dev/null
if($status) set nawk = awk
alias nawk $nawk
#
set infile   = ./mtz/rough_scaled.mtz
set outfile  = ./mtz/localscaled.mtz
set runfile  = ./runlist.txt
#
set tempfile = /tmp/jamesh/scaler_temp29967.lscale
#
# Defaults
set SPACING = 10
set CYCLES  = 100
set FILTER  = "#"
#
# read command-line arguments
goto Setup
############################
#
# usage: ./scripts/localscale.com [cycles] spacing [spacing] filter
#    eg: ./scripts/localscale.com 50 spacing 10 filter
#         means 50 cycles, 10-degree smoothing window and use the filter
#
############################
ReturnFromSetup:
######################################################################
#
# make SURE the mtz is sorted
#
######################################################################
rm -f ${tempfile}temp.mtz >& /dev/null
sortmtz HKLOUT ${tempfile}in.mtz << EOF
#VRSET -9E+38
H K L M/ISYM BATCH I SIGI
$infile
EOF

######################################################################
#
# Now run SCALA
#
######################################################################
rm -f ${tempfile}temp.mtz >& /dev/null
scala hklin ${tempfile}in.mtz  hklout ${tempfile}temp.mtz \
  scales   ${tempfile}scales \
  rogues   badspots.txt       \
  normplot ${tempfile}norm   \
  anomplot ${tempfile}anom  << EOF-scale
# save disk space
#NODUMP
#ANALYSE NOPLOT

title "Scaler Elves"

# resolution range
RESOLUTION 2.11 121.268

# reference data set (FP) is NOT merged
run 999 batch 1 reference

@$runfile

# global SCALA cards
SDCORR 0.75 2 0.02
INTENSITIES SCALE_PARTIALS 0.75 

#### treatment of partials ###### default, scale fulls only
#intensities partials		# use summed partials as fulls in scaling
#intensities scale_partials 0.8	# use extrapolated partials (by their fraction recorded) as fulls in scaling

#### type of scaling to be done ##########
#intensities anomalous                                    # scale I+ and I- as separate data sets
#scales batch bfactor off                                # one scale per frame
#scales batch bfactor on                                 # one scale and one B-factor per frame
#scales batch brotation spacing 10 bfactor on            # same, but B-factor smoothed over 10-degree window
#scales batch brotation spacing 10 bfactor anisotropic   # same, but anisotropic B-factor
#scales rotation spacing 10 bfactor on                   # scales smoothed over 10-degree window, B-factors follow SAME rule
#scales rotation spacing 10 detector 3 3                 # no B-factors, 9, smooth detector scales instead
#scales rotation spacing 10 secondary                    # spherical harmonics instead of detector scales and B-factors
scales rotation spacing $SPACING secondary               # Scaler Elves's opinion

# perhaps tie these down to prevent wild swings in scaling? 
#tie rotation 0.1
#tie detector 0.1
tie surface 0.001

cycles $CYCLES converge 0.01	# need to converge completely
${FILTER}filter 1.0e-4 0.01	# if convergence is oscillating

#reject byrun			# don't reject spots on comparison between runs

# print out statistics for I+/I- instead of Imean
anomalous

# do not merge (yet), but get rid of reference dataset
output separate


EOF-scale
######################################################################
if($status) then
    echo "SCALA has crashed."
    
    # check to see if, perhaps, the mtz was produced anyway
    echo "go" | mtzdump hklin ${tempfile}temp.mtz >& /dev/null
    if($status) then
	# file was no good
	rm -f ${tempfile}temp.mtz >& /dev/null
    endif
    if(-e ${tempfile}temp.mtz) then
	echo "but seems to have produced the output file with its last breath..."
	echo "YOU SHOULD MAKE SURE IT'S STILL OKAY! "
    endif
endif
if(! -e ${tempfile}temp.mtz) then
    echo "This usually happens when you have too many degrees of freedom in your run."
    echo "Try increasing the spacing of the smooth scales."
    echo "or decreasing the values in the tie cards."
    echo "Good luck! "
    echo ""
    # premature exit, allow user to see intermediate files
    exit 9
else
    # move file upon successful completion
    sortmtz hklout $outfile << EOF
#VRSET -9E+38
H K L M/ISYM BATCH I SIGI
${tempfile}temp.mtz
EOF
    rm -f ${tempfile}temp.mtz  >& /dev/null
endif

# clean up after scala
rm -f ${tempfile}in.mtz        >& /dev/null
rm -f ${tempfile}scales        >& /dev/null
rm -f ${tempfile}norm	        >& /dev/null
rm -f ${tempfile}anom	        >& /dev/null

exit

##############################################################################
Setup:
# process command-line arguments
set i = 1
while ( $i <= $#argv )
    # define the input file
    if(("$argv[$i]" =~ *.mtz)&&(-e "$argv[$i]")) then
	set infile = "$argv[$i]"
    else
    
	# may override default spacing
	if("$argv[$i]" =~ spac*) then
	    @ i = ( $i + 1)
	    if("$argv[$i]" =~ [1-9]*) set SPACING = "$argv[$i]"
	endif
    
	# may override default filtering
	if("$argv[$i]" == "filter") then
	    set FILTER = ""
	endif
    
	# raw numbers become cycle counts
	if(("$argv[$i]" =~ [1-9]*)&&("$argv[$i]" =~ *[0-9])) then
	    set CYCLES = "$argv[$i]"
	endif
    endif
    
    @ i = ( $i + 1)
end
#
# Run List passed down from Scaler Elves
if(! -e "$runfile" ) then
    # generate file containing list of scala runs
    cat << EOF-runs >! $runfile
# wavelength FP : 0.97973 A = 12654.9 eV
# 11000 +     1 to   100 in ../raw.mtz
run     1 11001 to 11100

EOF-runs
endif

goto ReturnFromSetup

