#! /bin/csh -f
#
#  Automatically generated script 
#  for mergeing one or all of FP
#
set nawk = /bin/awk
$nawk 'BEGIN{print}' >& /dev/null
if($status) set nawk = awk
alias nawk $nawk
#
set infile   = ./mtz/localscaled.mtz
set outfile  = merged.mtz
set runfile  = ./runlist.txt
#
set tempfile = /tmp/jamesh/scaler_temp29967.merge
#
# Default to mergeing ALL data together (useful? )
if(! $?wave) set wave     = "all"
set hires    = 2.11
#
# read command-line arguments
goto Setup
############################
#
# usage: ./scripts/merge.com [wavename] [high_RES] [SG]
#    eg: ./scripts/merge.com FP 2.11 P6122
#        means merge frames from FP to 2.11 A, in P6122
#
############################
ReturnFromSetup:
######################################################################
#
# Now run SCALA (to merge, not scale data)
#
######################################################################
rm -f ${tempfile}merged.mtz >& /dev/null
scala hklin $infile  hklout ${tempfile}merged.mtz \
  scales   ${tempfile}scales \
  rogues   badspots.$wave.txt     \
  normplot ${tempfile}norm \
  anomplot ${tempfile}anom  << EOF-scale
# save disk space
#NODUMP
#ANALYSE NOPLOT
# make sure scala doesnt do anyhting stupid like change the output filename
DNAME $wave
PNAME $wave

title "merged $wave from Scaler Elves"

# resolution range
RESOLUTION $hires 121.268

@${tempfile}runlist

# global SCALA cards
SDCORR 0.75 2 0.02
INTENSITIES SCALE_PARTIALS 0.75 

# treatment of partials	    # default, sum partials, and use result in analysis
#final fulls		    # use fulls only in analysis
#final scale_partials 0.5   # treat partials with fraction recorded > 0.5 as fulls in mergeing

onlymerge
noscale			    # should apply old scales!
#scales batch bfactor off

# print out statistics for I+/I- instead of Imean
# changing this to "off" will NOT affect output Imean
anomalous on

EOF-scale
##############################################################################
if($status) then
    echo "SCALA crashed! "
    # check to see if file was produced anyway
    echo "go" | mtzdump hklin ${tempfile}merged.mtz >& /dev/null
    if($status) then
	# file was no good
	rm -f ${tempfile}merged.mtz >& /dev/null
	set BAD
    else
	echo "...but seems to have produced the output data file before it did."
	echo "YOU SHOULD CHECK AND SEE IF THIS IS OKAY!!! "
    endif
endif

# clean up after scala
rm -f ${tempfile}runlist       >& /dev/null
rm -f ${tempfile}scales        >& /dev/null
rm -f ${tempfile}norm	        >& /dev/null
rm -f ${tempfile}anom	        >& /dev/null

truncate:
##############################################################################
# run truncate to make "F"s from "I"s
#
##############################################################################
truncate hklin ${tempfile}merged.mtz \
        hklout ${outfile} << EOF-trunc | tee ${tempfile}temp.log

title "merged $wave from Scaler Elves"

# overall resolution range 
RESOLUTION $hires 121.268
# resolution used to determine B factor
#rscale 4 2.8
#ranges 0.01

# needed to determine absolute scale
nresidue 373

# roll-off negative intensities with French-Wilson procedure
truncate yes
# changing this to "no" will NOT affect output Fs
anomalous yes
LABOUT  F=F  SIGF=SIGF DANO=DANO SIGDANO=SIGDANO ISYM=ISYM
EOF-trunc
##############################################################################
if($status) then
    # see if this is a fixable problem
    grep "Data beyond useful resolution limit" ${tempfile}temp.log >& /dev/null
    if($status) then
	# nope
	set BAD
    else
	# we know how to fix this (and there is no reason not to)
	cat ${tempfile}temp.log | nawk '/Wilson Plot/,/TRUNCATE/' |\
        nawk 'NF==10 && ! /[a-z]/{print $6}' |\
        tail -1 >! ${tempfile}res
        set hires = `cat ${tempfile}res`
        rm -f ${tempfile}res
	
        if("$hires" == "") then
            set hires = `awk '/Negative or zero mean I/ && $NF+0>0.1{print $NF+0}' ${tempfile}temp.log | tail -1`
        endif

	# go back, and try this again
	if("$hires" != "") then
	    set newRES = "$hires"
	    # prevent infinite loops if truncate is messed up
	    if(! $?retries) set retries = 0
	    @ retries = ( $retries + 1 )
	    if($retries <= 5) then
	        goto truncate
	    endif
	    set BAD
	endif
	set BAD
    endif
endif

# delete intermediate files
rm -f ${tempfile}merged.mtz	>& /dev/null
rm -f ${tempfile}temp.log	>& /dev/null
rm -f ${tempfile}reindexed.mtz >& /dev/null

##############################################################################
if(! $?BAD) then
    if($?newRES) echo "WARNING: output resolution reduced to $hires A"
    echo "merged $wave data is in ${outfile}"
else
    echo "mergeing failed!  See above for why."
    exit 9
endif

exit

##############################################################################
Setup:
#
# Run List passed down from Scaler Elves
if(! -e "$runfile" ) then
    # generate file containing list of scala runs
    cat << EOF-runs >! $runfile
# wavelength FP : 0.97973 A = 12654.9 eV
# 11000 +     1 to   100 in ../raw.mtz
run     1 11001 to 11100

EOF-runs
endif
#
# process command-line arguments
set newSG = ""
set axes  = ""
foreach arg ( $* )
    # define the input file
    if(("$arg" =~ *.mtz)&&(-e "$arg")) then
	set infile = "$arg"
    else
    
	# look for wavelength name
	set temp = `grep "wavelength" $runfile | grep " $arg " `
	if("$temp" != "") then
	    # must be the wavelength user wants
	    set wave = "$arg"
	else
	
	    # look for new resolution limit
	    set temp = `echo "$arg" | nawk '$1+0<10 && $1+0>0.4 {print $1+0}'`
	    if("$temp" != "") then
		# must be the resolution user wants
		set hires = "$temp"
	    endif
	
	endif
    endif
    
    # check for new space group
    if("$arg" =~ [PpCcIiFfRrHh][1-6]*) then
	# check for SGs listed in library (but not the screwy ones)
	set temp = `echo $arg | nawk '{print toupper($1)}'`
	if($?CLIBD) then
	    set temp = `nawk -v SG=$temp '$4 == SG && $1 < 500 {print $4}' $CLIBD/symop.lib | head -1`
	endif
	if("$temp" != "") then
	    set newSG = "$temp"
	endif
	
	set temp = `echo $arg | nawk '{print toupper($1)}'`
	# check for orthorhombic "pseudo-spacegroup" language
	if("$temp" == P2221) then
	    # P2221 with screw along longest axis
	    set axes  = "a b c"
	    set newSG = "P2221"
	    continue
	endif
	if("$temp" == P2212) then
	    # P2221 with screw along mid-length axis
	    set axes  = "b c a"
	    set newSG = "P2221"
	    continue
	endif
	if("$temp" == P2122) then
	    # P2221 with screw along shortest axis
	    set axes  = "c a b"
	    set newSG = "P2221"
	    continue
	endif
	
	if("$temp" == P21212) then
	    # P21212 with non-screw along longest axis
	    set axes  = "a b c"
	    set newSG = "P21212"
	    continue
	endif
	if("$temp" == P21221) then
	    # P21212 with non-screw along mid-length axis
	    set axes  = "b c a"
	    set newSG = "P21212"
	    continue
	endif
	if("$temp" == P22121) then
	    # P21212 with non-screw along shotest axis
	    set axes  = "c a b"
	    set newSG = "P21212"
	    continue
	endif
	    
	if("$temp" == C2221) then
	    # C2221 with screw along longest axis
	    set axes  = "a b c"
	    set newSG = "C2221"
	    continue
	endif
	# are these legal? 
	if("$temp" == C2212) then
	    # C2221 with screw along mid-length axis
	    set axes  = "b c a"
	    set newSG = "C2221"
	    continue
	endif
	if("$temp" == C2122) then
	    # C2221 with screw along shortest axis
	    set axes  = "c a b"
	    set newSG = "C2221"
	    continue
	endif
    endif
	
    # check for flipping of ambiguous axes
    if("$arg" == "flip") then
	# user requested "flip" of axes
	set FLIP
	# this should work for all P4x, P3x, P6x
	set REINDEX = "reindex k, h, -l"
	set message = "with a and b axes flipped"
	
	# are there any others?
    endif
    
    # what about cubic? 
end
#
# decide on new axis ordering (for asymmetric orthorhombics)
#if("$newSG" == "P222")    set axes = "a b c"
#if("$newSG" == "P212121") set axes = "a b c"
if("$axes" != "") then
    # get current axis ordering
    set CELL = `echo "head" | mtzdump hklin $infile | nawk '/Cell Dimensions/{getline;getline;print}'`
    # find out what the cannonical one would be
    # then decide how to go from current ordering to the desired one
    echo "$CELL" | nawk '{\
	# print out current axis order \
	print $1, "h"; print $2, "k"; print $3, "l"}' |\
    sort -n |\
    nawk '\
	# add cannonical axis names\
	NR==1{print $0, "a"} NR==2{print $0, "b"} NR==3{print $0, "c"}' |\
    nawk -v axes="$axes" 'BEGIN{split(axes, abc)} {\
	# write desired axis ordering in front of cannonical one \
	print abc[NR], $0}' |\
    sort |\
    cat >! ${tempfile}order    
    set REINDEX = "reindex "`nawk '{printf "%s", $3} NR~/[12]/{print ","}'  ${tempfile}order`

    # this should give us a mapping between any two orthorhombics
    
    set temp = `tail -1 ${tempfile}order | nawk '{print $NF}'`
    rm -f ${tempfile}order
    if(("$newSG" == "P2221")||("$newSG" == "C2221")) then
	set message = " with screw along $temp axis"
    endif
    if("$newSG" == "P21212") then
	set message = " with non-screw along $temp axis"
    endif
endif

if((! $?REINDEX) && ("$newSG" != "")) set REINDEX

if($?REINDEX) then
    if("$newSG" != "") set newSG = "SYMM $newSG"

    # reindex the input file to the new space group
    reindex HKLIN $infile HKLOUT ${tempfile}sortme.mtz << EOF-reindex
$newSG
$REINDEX
EOF-reindex
    
    # sort it too, for good measure
    sortmtz HKLIN ${tempfile}sortme.mtz HKLOUT ${tempfile}reindexed.mtz << EOF
#VRSET -9e+38
H K L M/ISYM BATCH I SIGI
EOF
    
    rm -f  ${tempfile}sortme.mtz
    # scale and merge this file
    set infile = ${tempfile}reindexed.mtz
endif
#
# Extract runs for the desired wavelength
cat $runfile |\
nawk -v wave="$wave" '$2=="wavelength"{p=0} $0 ~ " " wave " "{p=1} p{print}' |\
cat >! ${tempfile}runlist
#
set temp = `cat ${tempfile}runlist | wc -l`
if($temp == 0) then
    # default to all runs
    cat $runfile >! ${tempfile}runlist
endif

goto ReturnFromSetup

