#! /bin/csh -f -e
#
#   Prototype postref script
#
#
#	Takes standard unmerged SCALA output (using OUTPUT SEPARATE), 
#	reformats it for POSTREF, and postrefines each frame against
#	the overall, merged data set.
#	the results are output as a file suitable for reinput into
#	scala for mergeing (NOT scaling).
#
#	Caveats:
#	- The SDCORR option given below should match that used in the
#	  SCALA run.
#
#
set nawk = /bin/awk
$nawk 'BEGIN{print}' >& /dev/null
if($status) set nawk = awk
alias nawk $nawk
############################################################################
#
set infile   = ./mtz/localscaled.mtz
set outfile  = ./mtz/postrefined.mtz
#
set tempfile = /tmp/jamesh/scaler_temp29967.postref_temp.
#
############################################################################
# now scan command line
foreach arg ( $* )
    # specific input file
    if(-e "$arg") then
	if($arg =~ *.mtz) then
	    set infile = "$arg"
	endif
    endif
end
#
set MOSAICITY


Reformat:
############################################################################
# use scala to reformat data for postref
#
############################################################################
scala hklin $infile  hklout ${tempfile}reformatted.mtz \
  scales   ${tempfile}scales \
  rogues   badspots.txt       \
  normplot ${tempfile}norm   \
  anomplot ${tempfile}anom  << EOF-reformat
# global SCALA cards
SDCORR 0.75 2 0.02
INTENSITIES SCALE_PARTIALS 0.75 

anomalous
initial none
noscale
cycles 0
output postref
#final none
EOF-reformat
rm -f ${tempfile}scales >& /dev/null
rm -f ${tempfile}anom >& /dev/null
rm -f ${tempfile}norm >& /dev/null

PreSort:
############################################################################
# re-sort scaled data for postref
#
############################################################################
sortmtz HKLOUT ${tempfile}sorted.mtz << EOF-sort
#VRSET -9E+38
BATCH H K L M/ISYM
${tempfile}reformatted.mtz 
EOF-sort
rm -f ${tempfile}reformatted.mtz >& /dev/null

Rename:
# rename data to protect them from postref
# and use the sigma-corrected intensities
mtzutils hklin ${tempfile}sorted.mtz hklout ${tempfile}postrefme.mtz << EOF-utils
# postref changes ROT to range -180 to 180, which SCALA will not accept
#COLUMN_LABELS oldROT=ROT
# use SCALA's corrected sigmas
COLUMN_LABELS oldSIGI=SIGI oldSIGIPR=SIGIPR
COLUMN_LABELS SIGI=SIGIC   SIGIPR=SIGIPRC
EOF-utils
# bypass this? use SDFAC below? 
#mv ${tempfile}sorted.mtz ${tempfile}postrefme.mtz 
rm -f ${tempfile}sorted.mtz >& /dev/null


############################################################################
# examine file for lack of mosaicity (denzo data)
############################################################################
echo "BATCH" | mtzdump HKLIN ${tempfile}postrefme.mtz |\
nawk '$1 == "Mosaicity" {print $NF; if($NF+0 == 0) badmos=1} \
 badmos && /Phi angles/ {print $NF - $(NF-1)}' |\
nawk '{sum += $1;++n} END{if(n && badmos) print "MOSAICITY" sum/n}' |\
cat >! ${tempfile}mos
# set mosaicity to osc/2 if no mosaicity given
set MOSAICITY = `cat ${tempfile}mos`
rm -f ${tempfile}mos

Postref:
############################################################################
# now do postrefining
#
############################################################################
postref SUMMARY ${tempfile}summary \
          HKLIN ${tempfile}postrefme.mtz \
          HKLOUT ${tempfile}postrefed.mtz  << eof_postref

TITLE postreffing ${infile}

# no need for SDFAC if we use SIGIC
#SDCORR 0.75 0.02
#BEAM 0 0  0.0001 0
$MOSAICITY

# make sure this converges
REFINE NCYC 100 CONVRG 0.02
# default values for "REJECT" card:
REJECT 1000  -3 -3  0.5 3 0.0 0.0 10 

# refine all batches, individually, against the merged data
BATCH ALL
#CRYSTAL 1 to 77

OUTPUT ALL

END
eof_postref
rm -f ${tempfile}postrefme.mtz

# reformat output MTZ for input into SCALA again
mtzutils hklin ${tempfile}postrefed.mtz hklout ${tempfile}sortme.mtz << EOF-utils
# strip off incomplete line (will crash next postref run)
#EXCLUDE SIGSCALE SIGFRACTIONCALC IMEAN SIGIMEAN ISUM SIGISUM
# scala will crash if ROT is changed
#COLUMN_LABELS ROT=oldROT
# rename sigmas back to original values
COLUMN_LABELS SIGIC=SIGI   SIGIPRC=SIGIPR
COLUMN_LABELS SIGI=oldSIGI SIGIPR=oldSIGIPR
EOF-utils
#mv ${tempfile}postrefed.mtz ${tempfile}sortme.mtz >& /dev/null
rm -f ${tempfile}postrefed.mtz >& /dev/null

# re-sort for scala
sortmtz HKLOUT ${outfile} << EOF-sort
#VRSET -9e+38
H K L M/ISYM BATCH I SIGI
${tempfile}sortme.mtz
EOF-sort
# clean up
rm -f ${tempfile}sortme.mtz




# do xloggraph version of summary
cat << EOF-export >! ${tempfile}awkscript
#! /bin/nawk -f
#
#	Sumarize postref summary file in xloggraph format
#
#

# only read number-only lines
! /[a-z]/ && NF>1 {
    ++n; 
    
    # remember actual batch number
    batch[n] = \$1;
    
    # get cell parameters
    a[n]=\$2; a["avg"]+=a[n];
    b[n]=\$3; b["avg"]+=b[n]; 
    c[n]=\$4; c["avg"]+=c[n];
    A[n]=\$5; A["avg"]+=A[n];
    B[n]=\$6; B["avg"]+=B[n];
    G[n]=\$7; G["avg"]+=G[n];

    # get crystal orientation shifts
    phiX[n]= \$9; phiX["avg"] += phiX[n];
    phiY[n]=\$10; phiY["avg"] += phiY[n];
    phiZ[n]=\$11; phiZ["avg"] += phiZ[n];

    DphiX[n] = \$9-\$12;
    DphiY[n] = \$10-\$13;
    DphiZ[n] = \$11-\$14;
    
    # get mosaicity
    mos[n] = \$15; mos["avg"]+=mos[n];
    if(mos["max"] < mos[n]) mos["max"] = mos[n]
}

END{
    compute average values
    if(n)
    {
	a["avg"]=a["avg"]/n;
	b["avg"]=b["avg"]/n; 
	c["avg"]=c["avg"]/n;
	A["avg"]=A["avg"]/n;
	B["avg"]=B["avg"]/n;
	G["avg"]=G["avg"]/n;
    
	mos["avg"]=mos["avg"]/n;
    
	# print xloggraph plot header
	print " \$TABLE : - Elven Postref Plots:"
	print " \$GRAPHS:Unit Cell (deviations from mean):A:1, 3, 4, 5, 6, 7, 8:"
	print "        :Crystal Orientation (change during refinement):A:1, 9, 10, 11:"
	print "        :Crystal Slippage (orientation change during exposure):A:1, 12, 13, 14:"
	printf "        :Crystal Mosaicity (mean=%.3f max=%.3f):A:1, 15:\\n", mos["avg"], mos["max"]
	print " \$\$"
	print " frame batch A B C alpha beta gamma phiX phiY phiZ delta_phiX delta_phiY delta_phiZ mosaicity \$\$"
	print " \$\$"
    
	# compute deviates of the cell
	for(i=1;i<=n;++i)
	{
	    # compute deviates of the cell
	    a[i]-=a["avg"];
	    b[i]-=b["avg"]; 
	    c[i]-=c["avg"];
	    A[i]-=A["avg"];
	    B[i]-=B["avg"];
	    G[i]-=G["avg"];
	    
	    # print out interesting numbers as graphs
	    printf("%4d %6d %7.2f %7.2f %7.2f %7.2f %7.2f %7.2f   %7.2f %7.2f %7.2f   %7.2f %7.2f %7.2f   %7.3f\\n", \\
	    i, batch[i], a[i], b[i], c[i], A[i], B[i], G[i], \\
	    phiX[i], phiY[i], phiZ[i], DphiX[i], DphiY[i], DphiZ[i], mos[i]);
	}

	print "\$\$"
	
	printf "Average cell is  : %.2f %.2f %.2f %.2f %.2f %.2f\\n", a["avg"],  b["avg"],  c["avg"],  A["avg"],  B["avg"],  G["avg"];
	printf "Average mosaicity: %.3f\\n", mos["avg"];
	printf "Maximum mosaicity: %.3f\\n", mos["max"];
    }    
}
EOF-export

# dump results to stdout
nawk -f ${tempfile}awkscript  ${tempfile}summary

# finish cleaning up
rm -f ${tempfile}awkscript 
rm -f ${tempfile}summary

##########################################################################
if(-e $outfile) then
    echo "$outfile now contains a postrefined version of $infile "
endif
##########################################################################

exit

